@@ -361,7 +361,7 @@ class Test_Yfc_Interface(Test_Base):
         start_day_str = "2022-06-13"
         end_day_str = "2022-06-18"
 
-        for aa in [False,True]:
+        for aa in [False, True]:
             df_yf = dat_yf.history(start=start_day_str, end=end_day_str, auto_adjust=aa, back_adjust=False)
             df_yfc = self.nze_dat.history(start=start_day_str, end=end_day_str, adjust_divs=aa)
 
@@ -371,15 +371,16 @@ class Test_Yfc_Interface(Test_Base):
                 elif c == "Adj Close" and not c in df_yfc.columns:
                     continue
                 try:
-                    if aa:
-                        self.assertTrue(np.isclose(df_yf[c].values, df_yfc[c].values, rtol=1e-10).all())
+                    if aa or "Adj" in c:
+                        # self.assertTrue(np.isclose(df_yf[c].values, df_yfc[c].values, rtol=1e-10).all())
+                        self.assertTrue(np.isclose(df_yf[c].values, df_yfc[c].values, rtol=5e-6).all())
                     else:
                         self.assertTrue(np.equal(df_yf[c].values, df_yfc[c].values).all())
                 except:
                     print("df_yf:")
-                    print(df_yf)
+                    print(df_yf[[c]])
                     print("df_yfc:")
-                    print(df_yfc)
+                    print(df_yfc[[c]])
                     print("aa={}, c={}".format(aa, c))
                     raise
 
@@ -463,9 +464,12 @@ class Test_Yfc_Interface(Test_Base):
         tkrs = self.tkrs
         periods = [p for p in yfcd.Period]
         #
+        # tkrs = ["BHP.AX"]
+        # periods = [yfcd.Period.Max]
         for tkr in tkrs:
             dat_yf = yf.Ticker(tkr, session=self.session)
             for p in periods:
+                # print(tkr, p)
                 self.tempCacheDir.cleanup() ; self.tempCacheDir = tempfile.TemporaryDirectory()
                 yfcm.SetCacheDirpath(self.tempCacheDir.name)
                 dat_yfc = yfc.Ticker(tkr, session=self.session)
