@@ -2,6 +2,7 @@
 
 from .yfc_dat import Period, Interval
 from .yfc_ticker import Ticker, verify_cached_tickers_prices
+from .yfc_logging import EnableLogging, DisableLogging
 
 from .yfc_upgrade import _move_cache_dirpath
 _move_cache_dirpath()
@@ -15,8 +16,9 @@ _sanitise_prices()
 _separate_events_from_prices()
 _fix_dividend_adjust()
 
-from .yfc_upgrade import _fix_listing_date
+from .yfc_upgrade import _fix_listing_date, _upgrade_divs_splits_supersede
 _fix_listing_date()
+_upgrade_divs_splits_supersede()
 
 # __all__ = ['Ticker', 'Period', 'Interval']
 
