@@ -348,7 +348,6 @@ def ReadCacheMetadata(ticker, objectName, key):
         pkData = _ReadPackedData(ticker, objectName)
         if (pkData is not None) and (objectName in pkData):
             objData = pkData[objectName]
-            # data    = objData["data"]
             md      = objData["metadata"] if "metadata" in objData else None
     else:
         d = _ReadData(ticker, objectName)
@@ -370,13 +369,22 @@ def WriteCacheMetadata(ticker, objectName, key, value):
     if IsObjectInPackedData(objectName):
         return WriteCachePackedMetadata(ticker, objectName, key, value)
 
+    if verbose:
+        if value is None:
+            print(f"WriteCacheMetadata({ticker}, {objectName}, {key}) deleting")
+        else:
+            print(f"WriteCacheMetadata({ticker}, {objectName}, {key}) storing")
+
     d = _ReadData(ticker, objectName)
     if d is None:
         raise Exception("'{}/{}' not in cache, cannot add metadata".format(ticker, objectName))
 
     if "metadata" in d:
-        d["metadata"][key] = value
-    else:
+        if value is None:
+            del d["metadata"][key]
+        else:
+            d["metadata"][key] = value
+    elif value is not None:
         d["metadata"] = {key: value}
 
     if verbose:
@@ -391,6 +399,7 @@ def WriteCacheMetadata(ticker, objectName, key, value):
         with open(fp, 'wb') as outData:
             pickle.dump(d, outData, 4)
 
+
 def WriteCachePackedMetadata(ticker, objectName, key, value):
     if not IsObjectInPackedData(objectName):
         return WriteCacheMetadata(ticker, objectName)
